<?php

declare(strict_types=1);

namespace Drupal\samlauth_group\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\TypedData\TypedDataManagerInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\samlauth\Event\SamlauthEvents;
use Drupal\samlauth\Event\SamlauthUserSyncEvent;

/**
 * Synchronize the SAML attributes into group memberships.
 */
final class SamlauthGroupSubscriber implements EventSubscriberInterface {

  /**
   * Name of the configuration object containing the setting used by this class.
   */
  const CONFIG_OBJECT_NAME = 'samlauth_group.membership_mappings';

  /**
   * The configuration factory service.
   *
   * We're doing $configFactory->get() all over the place to access our
   * configuration, which is actually a little more efficient than storing the
   * config object in a variable in this class.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The typed data manager service.
   *
   * @var \Drupal\Core\TypedData\TypedDataManagerInterface
   */
  protected $typedDataManager;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a SamlauthGroupSubscriber object.
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
    LoggerInterface $logger,
    TypedDataManagerInterface $typedDataManager,
    EntityTypeManagerInterface $entityTypeManager,
  ) {
    $this->configFactory = $configFactory;
    $this->logger = $logger;
    $this->typedDataManager = $typedDataManager;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Saves configured group membership mapping based on SAML attribute values.
   *
   * @param \Drupal\samlauth\Event\SamlauthUserSyncEvent $event
   *   The event being dispatched.
   */
  public function onUserSync(SamlauthUserSyncEvent $event): void {
    $account = $event->getAccount();

    // Skip if the account is blocked.
    if ($account->isBlocked()) {
      return;
    }

    $config = $this->configFactory->get(static::CONFIG_OBJECT_NAME);
    $mappings = $config->get('membership_mappings');
    $validation_errors = [];
    if (is_array($mappings)) {
      foreach ($mappings as $mapping) {
        // If the attribute name is invalid, or the field does not exist, spam
        // the logs on every login until the mapping is fixed.
        if (empty($mapping['attribute_name']) || !is_string($mapping['attribute_name'])) {
          $this->logger->warning('Invalid SAML attribute %attribute detected in mapping; the mapping must be fixed.');
        }
        elseif (empty($mapping['attribute_value']) || !is_string($mapping['attribute_value'])) {
          $this->logger->warning('Invalid SAML attribute value for attribute named %attribute; the mapping must be fixed.', ['%attribute' => $mapping['attribute_name']]);
        }

        // Check that either the group ID exists or the group type exists.
        if (empty($mapping['group_id']) && empty($mapping['group_type'])) {
          $this->logger->warning('Missing group ID and group type in mapping; the mapping must be fixed.', ['%group_id' => $mapping['group_id']]);
        }

        // Skip silently if the attribute value doesn't match.
        if ($this->getAttribute($mapping['attribute_name'], $event->getAttributes()) !== $mapping['attribute_value']) {
          continue;
        }

        // Add the user to the group(s).
        if (!empty($mapping['group_id'])) {
          try {
            /** @var \Drupal\group\Entity\GroupInterface */
            $group = $this->entityTypeManager->getStorage('group')->load($mapping['group_id']);
            if ($group) {
              $group->addMember($account);
            }
            else {
              $validation_errors[] = $mapping['group_id'];
            }
          }
          catch (\Exception $e) {
            $validation_errors[] = $mapping['group_id'];
            $this->logger->error('Failed to add user %uid to group %gid: %error', [
              '%uid' => $account->id(),
              '%gid' => $mapping['group_id'],
              '%error' => $e->getMessage(),
            ]);
          }
        }
        elseif (!empty($mapping['group_type'])) {
          // Load each group of the given type and add the user to it.
          $groups = $this->entityTypeManager->getStorage('group')->loadByProperties(['type' => $mapping['group_type']]);

          /** @var \Drupal\group\Entity\GroupInterface */
          foreach ($groups as $group) {
            try {
              $group->addMember($account);
            }
            catch (\Exception $e) {
              $validation_errors[] = $group->id();
              $this->logger->error('Failed to add user %uid to group %gid: %error', [
                '%uid' => $account->id(),
                '%gid' => $group->id(),
                '%error' => $e->getMessage(),
              ]);
            }
          }
        }
      }
    }
    // @todo add support for revoking group membership.
    // @todo add support for adding/revoking group roles.
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[SamlauthEvents::USER_SYNC][] = ['onUserSync'];
    return $events;
  }

  /**
   * Returns value from a SAML attribute; logs anything strange.
   *
   * This is suitable for single-value attributes. For multi-value attributes,
   * we log a debug message to make clear we're dropping data (because this
   * indicates that the site owner may need to take care of getting more
   * sophisticated path mapping code).
   *
   * @param string $name
   *   The name of a SAML attribute.
   * @param array $attributes
   *   The complete set of SAML attributes in the assertion. (The attributes
   *   can currently be duplicated, keyed both by their name and friendly name.)
   *
   * @return mixed|null
   *   The SAML attribute value; NULL if the attribute value was not found.
   */
  protected function getAttribute($name, array $attributes) {
    $value = NULL;
    if (isset($attributes[$name])) {
      // Log everything unexpected about the format of the attributes. Use
      // debug() because we're not sure if the site owner would be able to fix
      // things.
      if (!is_array($attributes[$name])) {
        $this->logger->debug('SAML attribute %name has a non-array value; this points to a coding error somewhere (since the SAML standard seems to mandate this).', ['%name' => $name]);
      }
      elseif ($attributes[$name]) {
        if (count($attributes[$name]) > 1) {
          $this->logger->debug('SAML attribute %name has multiple values; we only support using the first one: @values.', [
            '%name' => $name,
            '@values' => function_exists('json_encode') ? json_encode($attributes[$name]) : var_export($attributes[$name], TRUE),
          ]);
        }
        if (!isset($attributes[$name][0])) {
          $value = reset($attributes[$name]);
          $this->logger->debug("SAML attribute %name's one-element array value has non-zero key %key, which points to a coding error somewhere; even though we are using the value, we're not sure if that's right.", ['%name' => $name]);
        }
        else {
          $value = $attributes[$name][0];
        }
      }
    }
    return $value;
  }

}
