<?php

namespace Drupal\samlauth_group\Form;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\samlauth_group\EventSubscriber\SamlauthGroupSubscriber;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\group\Entity\GroupRoleInterface;

/**
 * Form for adding a mapped SAML attribute -> group membership.
 */
class SamlauthGroupMembershipMappingEditForm extends FormBase {


  /**
   * The entity field manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The entity type bundle info manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityTypeBundleInfoManager;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * SamlauthMappingEditForm constructor.
   *
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager service.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle info manager service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   */
  public function __construct(EntityFieldManagerInterface $entity_field_manager, EntityTypeBundleInfoInterface $entity_type_bundle_info, EntityTypeManagerInterface $entity_type_manager) {
    $this->entityFieldManager = $entity_field_manager;
    $this->entityTypeBundleInfoManager = $entity_type_bundle_info;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_field.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'samlauth_group_membership_edit_form';
  }

  /**
   * Form for adding or editing a mapping.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param int $mapping_id
   *   (optional) The numeric ID of the mapping.
   *
   * @return array
   *   The form structure.
   */
  public function buildForm(array $form, FormStateInterface $form_state, $mapping_id = NULL) {
    $mappings = $this->configFactory()->getEditable(SamlauthGroupSubscriber::CONFIG_OBJECT_NAME)->get('membership_mappings');

    // Create the list options for the group types.
    $group_types = $this->entityTypeBundleInfoManager->getBundleInfo('group');
    $group_types_options = [];
    $default_group = NULL;
    if (!empty($mappings[$mapping_id]['group_id'])) {
      // In case the group was deleted or not found allow the form to work
      // anyways.
      try {
        $default_group = $this->entityTypeManager->getStorage('group')->load($mappings[$mapping_id]['group_id']);
      }
      catch (\Exception $e) {
        $this->messenger()->addError($this->t('The group with ID @id could not be loaded.', ['@id' => $mappings[$mapping_id]['group_id']]));
      }
    }
    // Set the list of available group types. Default is none.
    $group_types_options[''] = '-- ' . $this->t('None') . ' --';
    foreach ($group_types as $group_type => $group_type_info) {
      $group_types_options[$group_type] = $group_type_info['label'];
    }

    $form['attribute_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('SAML Attribute'),
      '#description' => $this->t('The name of the SAML attribute to compare with the given value.'),
      '#required' => TRUE,
      '#default_value' => $mappings[$mapping_id]['attribute_name'] ?? NULL,
    ];

    $form['attribute_value'] = [
      '#type' => 'textfield',
      '#title' => $this->t('SAML Attribute Value'),
      '#description' => $this->t('The value of the SAML attribute required to match to this membership.'),
      '#required' => TRUE,
      '#default_value' => $mappings[$mapping_id]['attribute_value'] ?? NULL,
    ];

    // Autocomplete field to select the group to map the user to.
    $form['group_id'] = [
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('Group'),
      '#description' => $this->t('The group you want to add the user to.'),
      '#required' => FALSE,
      '#default_value' => $default_group,
      '#target_type' => 'group',
      "#states" => [
        "visible" => [
          ":input[name='group_type']" => ["value" => ""],
        ],
        'required' => [
          ":input[name='group_type']" => ["value" => ""],
        ],
        'resettable' => [
          ":input[name='group_type']" => ["value" => ""],
        ],
      ],
      '#ajax' => [
        'callback' => '::groupRolesAjaxCallback',
        'wrapper' => 'group-roles-wrapper',
        'event' => 'autocompleteclose',
      ],
    ];

    $form["group_type"] = [
      '#type' => 'select',
      '#title' => $this->t('Group Type'),
      '#description' => $this->t('The type of group you want to create a membership to. This option will map the given user to all groups of the given type (e.g. department mamanger).'),
      '#required' => FALSE,
      '#options' => $group_types_options,
      '#default_value' => $mappings[$mapping_id]['group_type'] ?? NULL,
      "#states" => [
        "enabled" => [
          ":input[name='group_id']" => ["value" => ""],
        ],
        'required' => [
          ":input[name='group_id']" => ["value" => ""],
        ],
        "visible" => [
          ":input[name='group_id']" => ["value" => ""],
        ],
      ],
      '#ajax' => [
        'callback' => '::groupRolesAjaxCallback',
        'wrapper' => 'group-roles-wrapper',
        'event' => 'change',
      ],
    ];

    $gvalues = $form_state->getValue(['group_type']);
    $ivalues = $form_state->getValue(['group_id']);

    // Limit the list of selectable roles based on the selected group/type.
    if ($selected_group = $form_state->getValue('group_id') ?? $mappings[$mapping_id]['group_id'] ??  NULL) {
      // Get the group type of the selected group.
      $selected_group = $this->entityTypeManager->getStorage('group')->load($selected_group);
      $selected_group_types[] = $selected_group->bundle();
    }
    if ($selected_group_types = $form_state->getValue('group_type') ?? $mappings[$mapping_id]['group_type'] ?? NULL) {
      $selected_group_types = $selected_group_types;
    }

    $group_roles = $this->entityTypeManager->getStorage('group_role')->loadByProperties(['group_type' => $selected_group_types]);
    $group_roles_options = [];
    foreach ($group_roles as $group_role) {
      $group_roles_options[$group_role->id()] = $group_role->label();
    }
    $form['group_roles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Group Roles'),
      '#description' => $this->t('The roles the user should have in the group. Leave blank to use default member role.'),
      '#options' => $group_roles_options,
      '#default_value' => $mappings[$mapping_id]['group_roles'] ?? [],
      '#prefix' => '<div id="group-roles-wrapper">',
      '#suffix' => '</div>',
    ];

    // Add this value so we know if it's an add or an edit.
    $form['mapping_id'] = [
      '#type' => 'value',
      '#value' => $mapping_id,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    ];

    return $form;
  }

  /**
   * AJAX callback for the group roles field.
   *
   * Return the group roles field with the list of roles set based on the
   * group type(s).
   */
  public function groupRolesAjaxCallback(array &$form, FormStateInterface $form_state) {
    return $form['group_roles'];
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Check if at least one of group_type or group_id is set.
    if (empty($form_state->getValue('group_type')) && empty($form_state->getValue('group_id'))) {
      $form_state->setErrorByName('group_type', $this->t('You must select a group type or enter a group ID.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory()->getEditable(SamlauthGroupSubscriber::CONFIG_OBJECT_NAME);
    $mappings = $config->get('membership_mappings');

    $new_mapping = [
      'attribute_name' => $form_state->getValue('attribute_name'),
      'attribute_value' => $form_state->getValue('attribute_value'),
      'group_id' => $form_state->getValue('group_id'),
      'group_roles' => $form_state->getValue('group_roles'),
    ];

    // Only set the group type if the group id is empty.
    if (empty($new_mapping['group_id'])) {
      $new_mapping['group_type'] = $form_state->getValue('group_type');
    }
    else {
      $new_mapping['group_type'] = '';
    }

    $mapping_id = $form_state->getValue('mapping_id');
    if (!is_array($mappings)) {
      $mappings = [];
    }
    if ($mapping_id !== NULL) {
      $mappings[$mapping_id] = $new_mapping;
    }
    else {
      $mappings[] = $new_mapping;
    }
    $config->set('membership_mappings', $mappings)->save();

    $form_state->setRedirect('samlauth_group.membership_list');
  }

}
