<?php

namespace Drupal\samlauth_group\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\group\Entity\GroupInterface;
use Drupal\samlauth_group\EventSubscriber\SamlauthGroupSubscriber;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Displays the list of attribute-field mappings; edits related configuration.
 */
class SamlauthGroupMembershipMappingListForm extends ConfigFormBase {

  /**
   * Entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);

    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->renderer = $container->get('renderer');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [SamlauthGroupSubscriber::CONFIG_OBJECT_NAME];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'samlauth_group_memberhips_edit_form';
  }

  /**
   * Form for adding or editing a mapping.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param int $mapping_id
   *   (optional) The numeric ID of the mapping.
   *
   * @return array
   *   The form structure.
   */
  public function buildForm(array $form, FormStateInterface $form_state, $mapping_id = NULL) {
    $config = $this->config(SamlauthGroupSubscriber::CONFIG_OBJECT_NAME);

    // The bulk of this page is not a form at all, but a table. We're putting
    // that on the same page as the form options, because we have only two
    // checkboxes - which govern behavior related to the total of those table
    // rows. If this configuration form somehow grows, we'll split the table +
    // form off into separate pages/routes.
    $mappings = $config->get('membership_mappings');
    $form = $this->listMappings(is_array($mappings) ? $mappings : []);

    return parent::buildForm($form, $form_state);

  }

  /**
   * Returns the list of attribute-field mappings.
   *
   * @param array $mappings
   *   The attribute-field mappings.
   *
   * @return array
   *   A renderable content array.
   */
  public function listMappings(array $mappings) {
    $output['table'] = [
      '#theme' => 'table',
      '#header' => [
        $this->t('SAML Attribute'),
        $this->t('SAML Attribute Value'),
        $this->t('Group'),
        $this->t('Group Type'),
        $this->t('Operations'),
      ],
      '#sticky' => TRUE,
      '#empty' => $this->t("There are no mappings. You can add one using the link above."),
    ];

    if ($mappings) {
      foreach ($mappings as $id => $mapping) {
        $operations = [
          '#type' => 'dropbutton',
          '#links' => [
            'edit' => [
              'title' => $this->t('edit'),
              'url' => Url::fromRoute('samlauth_group.edit_membership', ['mapping_id' => $id]),
            ],
            'delete' => [
              'title' => $this->t('delete'),
              'url' => Url::fromRoute('samlauth_group.delete_membership', ['mapping_id' => $id]),
            ],
          ],
        ];

        // Try to load the group entity to show the name. Otherwise use the
        // group ID.
        $group_name = $mapping['group_id'];
        try {
          if (!empty($mapping['group_id'])) {
            $group = $this->entityTypeManager->getStorage('group')->load($mapping['group_id']);

            if ($group instanceof GroupInterface) {
              $group_name = $group->label();
              // Set the group name to output as a link to the group itself.
              $group_name = $group->toLink($group_name);
            }
            else {
              $group_name = $this->t('Group %id Missing', ['%id' => $mapping['group_id']]);
            }
          }
          else {
            $group_name = $this->t('N/A');
          }
        }
        catch (\Exception $e) {
          // Notify the user that a group could not be loaded.
          $this->messenger()->addError($this->t('The group with ID @id could not be loaded.', ['@id' => $mapping['group_id']]));
        }

        $output['table']['#rows'][$id] = [
          $mapping['attribute_name'],
          $mapping['attribute_value'],
          $group_name,
          $mapping['group_type'],
          $this->renderer->render($operations),
        ];
      }
    }

    return $output;
  }

}
